'use strict';

const Promise = require('bluebird');
const AWS = require('aws-sdk');


// This establishes a private namespace.
const namespace = new WeakMap();
function p(object) {
  if (!namespace.has(object)) namespace.set(object, {});
  return namespace.get(object);
}



/**
 *
 */
module.exports = class LambdaInvoker {
  constructor(config) {
    config = config || {};

    if (!config.lambdaARN) throw new Error('lambdaARN must be provided.');
    p(this).lambdaARN = config.lambdaARN;

    p(this).client = new AWS.Lambda({
      region: config.region,
      credentials: new AWS.Credentials({
        accessKeyId: config.accessKeyId,
        secretAccessKey: config.secretAccessKey
      })
    });
  }

  invoke(payload, config) {
    config = config || {};
    return Promise.resolve(p(this).client.invoke({
      FunctionName: p(this).lambdaARN,
      InvocationType: 'RequestResponse',
      LogType: config.logs ? 'Tail' : 'None',
      Payload: JSON.stringify(payload) 
    }).promise())
    .then((response) => {
      let parsed = JSON.parse(response.Payload);
      // In case stringified, will need to be parsed again.
      try { parsed = JSON.parse(parsed); }
      catch(err) {}
      
      if (parsed.errorMessage) return Promise.reject(new Error(parsed.errorMessage));
      if (config.log) {
        const rawLog = response.LogResult;
        let decoded;
        try {
          decoded = Buffer.from(rawLog, 'base64').toString('ascii');
        }
        catch(err) {
          decoded = (new Buffer(rawLog, 'base64')).toString('ascii');
        }
        return {logs, result: parsed};
      }
      return parsed;
    });
  }
}
