'use strict';

const Promise = require('bluebird');
const AWS = require('aws-sdk');
const Deferrari = require('deferrari');

const STATE = {
  INITIALIZED: 'initialized'
};

const client = {
  S3: require('./s3')
  // Other store types to come.
};


// This establishes a private namespace.
const namespace = new WeakMap();
function p(object) {
  if (!namespace.has(object)) namespace.set(object, {});
  return namespace.get(object);
}



/**
 *
 */
module.exports = class Store {
  constructor(config) {
    config = config || {};

    p(this).deferrari = new Deferrari({Promise});

    if (config.S3) {
      p(this).client = new client.S3(config.S3)
    }

    p(this).client.initialize()
    .then(response => p(this).deferrari.resolve(STATE.INITIALIZED, response));
  }

  /**
   * Get value at key.
   */
  get(key) {
    return p(this).deferrari.deferUntil(STATE.INITIALIZED)
    .then(() => p(this).client.get(key));
  }

  /**
   * Upsert value at key.
   */
  upsert(key, value) {
    return p(this).deferrari.deferUntil(STATE.INITIALIZED)
    .then(() => p(this).client.upsert(key, value));
  }
}
